@@ -5,7 +5,7 @@ from transformers.modeling_bert import BertOnlyMLMHead
 
 from dnn_cool.losses import LanguageModelCrossEntropyLoss, ReducedPerSample
 from dnn_cool.missing_values import all_correct
-from dnn_cool.task_flow import Task
+from dnn_cool.tasks import Task
 
 
 @dataclass()
