@@ -1,25 +1,26 @@
 from collections import OrderedDict
-from shutil import copyfile
-from typing import Iterator, Mapping
 from dataclasses import dataclass, field
-from functools import partial
+from functools import partial, cached_property
 from pathlib import Path
+from shutil import copyfile
 from time import time
-from typing import Dict, Tuple, Callable, Optional, Any, Union, List
-from torch import nn
+from typing import Dict, Tuple, Callable, Optional, Any, Union, List, Sized
+from typing import Iterator, Mapping
 
 import numpy as np
 import torch
 from catalyst.dl import SupervisedRunner, EarlyStoppingCallback, InferCallback, State, Callback
 from catalyst.utils import load_checkpoint, unpack_checkpoint, any2device
+from torch import nn
 from torch import optim
 from torch.optim.lr_scheduler import ReduceLROnPlateau
 from torch.optim.optimizer import Optimizer
 from torch.utils.data import DataLoader, Dataset
 
-from dnn_cool.catalyst_utils import InterpretationCallback, TensorboardConverters, ReplaceGatherCallback
-from dnn_cool.deployment import unbind_task_labels
-from dnn_cool.utils import TransformedSubset, train_test_val_split
+from dnn_cool.catalyst_utils import InterpretationCallback, TensorboardConverters, ReplaceGatherCallback, \
+    TensorboardConverter
+from dnn_cool.tasks import TaskFlow, TaskFlowForDevelopment
+from dnn_cool.utils import TransformedSubset, train_test_val_split, load_model_from_export
 
 
 @dataclass
@@ -117,17 +118,130 @@ class InferDictCallback(InferCallback):
         }
 
 
+def load_inference_results_from_directory(logdir):
+    out_dir = logdir / 'infer'
+    out_dir.mkdir(exist_ok=True)
+    res = {}
+    for key in ['logits', 'targets', 'interpretations']:
+        file_path = out_dir / f'{key}.pkl'
+        if file_path.exists():
+            res[key] = torch.load(out_dir / f'{key}.pkl')
+    return res
+
+
+class DnnCoolRunnerView:
+
+    def __init__(self, full_flow: TaskFlow, model: nn.Module,
+                 project_dir: Union[str, Path], runner_name: str):
+        self.project_dir = Path(project_dir)
+        self.full_flow = full_flow
+        self.model = model
+        self.runner_name = runner_name
+        self.default_logdir_name = f'./logdir_{runner_name}'
+
+    def best(self) -> nn.Module:
+        return self.load_model_from_checkpoint('best')
+
+    def last(self) -> nn.Module:
+        return self.load_model_from_checkpoint('last')
+
+    def from_epoch(self, i) -> nn.Module:
+        return self.load_model_from_checkpoint(f'train.{i}')
+
+    def load_model_from_export(self, out_directory: Union[str, Path]) -> nn.Module:
+        return load_model_from_export(self.model, self.full_flow, out_directory)
+
+    def load_model_from_checkpoint(self, checkpoint_name) -> nn.Module:
+        model = self.model
+        logdir = self.project_dir / self.default_logdir_name
+        checkpoint_path = str(logdir / 'checkpoints' / f'{checkpoint_name}.pth')
+        model.load_state_dict(torch.load(checkpoint_path)['model_state_dict'])
+        thresholds_path = logdir / 'tuned_params.pkl'
+        if not thresholds_path.exists():
+            return model
+        tuned_params = torch.load(thresholds_path)
+        self.full_flow.get_decoder().load_tuned(tuned_params)
+        return model
+
+    def load_tuned_params(self) -> Optional[Dict]:
+        logdir = self.project_dir / self.default_logdir_name
+        thresholds_path = logdir / 'tuned_params.pkl'
+        if not thresholds_path.exists():
+            return None
+        return torch.load(thresholds_path)
+
+    def load_inference_results(self) -> Dict:
+        logdir = self.project_dir / self.default_logdir_name
+        return load_inference_results_from_directory(logdir)
+
+    def load_train_test_val_split(self):
+        return read_split(self.project_dir / self.default_logdir_name)
+
+    @cached_property
+    def train_test_val_indices(self):
+        return self.load_train_test_val_split()
+
+    @cached_property
+    def inference_results(self):
+        return self.load_inference_results()
+
+    @cached_property
+    def evaluation_df(self):
+        import pandas as pd
+        return pd.read_csv(self.project_dir / self.default_logdir_name / 'evaluation.csv')
+
+    def summarize_loss_values(self, loader_name, task_name):
+        interpretations = self.inference_results['interpretations'][loader_name]
+        loss_values = interpretations[task_name]
+        loss_local_indices = interpretations[f'indices|{task_name}']
+        loader_idx = ['infer', 'test', 'valid'].index(loader_name)
+        global_loader_indices = self.train_test_val_indices[loader_idx]
+        mask = loss_local_indices >= 0
+        logits = self.inference_results['logits'][loader_name][task_name][loss_local_indices[mask]]
+        task = self.full_flow.get_all_children()[task_name]
+        logits = torch.tensor(logits)
+        activated = task.get_activation()(logits) if task.get_activation() is not None else logits
+        decoded = task.get_decoder()(activated) if task.get_decoder() is not None else activated
+        return {
+            'global_idx': global_loader_indices[loss_local_indices[mask]],
+            'loss_values': loss_values[mask],
+            'targets': self.inference_results['targets'][loader_name][task_name][loss_local_indices[mask]],
+            'activated': activated,
+            'decoded': decoded.numpy(),
+            'logits': logits.numpy(),
+            'task': task
+        }
+
+    def worst_examples(self, loader_name, task_name, n):
+        return self.extremal_examples(loader_name, task_name, n, -1)
+
+    def best_examples(self, loader_name, task_name, n):
+        return self.extremal_examples(loader_name, task_name, n, 1)
+
+    def extremal_examples(self, loader_name, task_name, n, mult):
+        res = self.summarize_loss_values(loader_name, task_name)
+        sorter = (mult * res['loss_values']).argsort()
+        top_n_idx = res['global_idx'][sorter[:n]]
+        return top_n_idx, res
+
+
+def batch_to_device(batch, device) -> Mapping[str, torch.Tensor]:
+    return any2device(batch, device)
+
+
 class DnnCoolSupervisedRunner(SupervisedRunner):
 
-    def __init__(self, project, model,
+    def __init__(self, model: nn.Module,
+                 full_flow: TaskFlowForDevelopment,
+                 project_dir: Union[str, Path],
+                 runner_name: str,
+                 tensoboard_converters: TensorboardConverter,
                  early_stop: bool = True,
                  balance_dataparallel_memory: bool = False,
-                 runner_name=None,
-                 train_test_val_indices=None,
-                 perform_conversion=True):
-        self.task_flow = project.get_full_flow()
+                 train_test_val_indices: Tuple[np.ndarray, np.ndarray, np.ndarray] = None):
+        self.task_flow = full_flow
 
-        self.default_criterion = self.task_flow.get_loss()
+        self.default_criterion = self.task_flow.get_criterion()
         self.balance_dataparallel_memory = balance_dataparallel_memory
 
         self.default_callbacks = []
@@ -136,22 +250,21 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         self.default_callbacks.extend(self.default_criterion.catalyst_callbacks())
         self.default_optimizer = partial(optim.AdamW, lr=1e-4)
         self.default_scheduler = ReduceLROnPlateau
-        self.project_dir: Path = project.project_dir
+        self.project_dir: Path = Path(project_dir)
         self.project_dir.mkdir(exist_ok=True)
-        runner_name = f'{self.task_flow.get_name()}_{time()}' if runner_name is None else runner_name
         self.default_logdir = f'./logdir_{runner_name}'
 
         if early_stop:
             self.default_callbacks.append(EarlyStoppingCallback(patience=5))
 
         (self.project_dir / self.default_logdir).mkdir(exist_ok=True)
-        if perform_conversion:
-            if train_test_val_indices is None:
-                train_test_val_indices = project_split(project.df, self.project_dir / self.default_logdir)
-            else:
-                save_split(self.project_dir / self.default_logdir, train_test_val_indices)
+        if train_test_val_indices is None:
+            n = len(self.task_flow.get_dataset())
+            train_test_val_indices = runner_split(n, self.project_dir / self.default_logdir)
+        else:
+            save_split(self.project_dir / self.default_logdir, train_test_val_indices)
         self.train_test_val_indices = train_test_val_indices
-        self.tensor_loggers = project.converters.tensorboard_converters
+        self.tensor_loggers = tensoboard_converters
         super().__init__(model=model)
 
     def train(self, *args, **kwargs):
@@ -166,8 +279,7 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         if 'scheduler' not in kwargs:
             kwargs['scheduler'] = self.default_scheduler(kwargs['optimizer'])
 
-        if 'logdir' not in kwargs:
-            kwargs['logdir'] = self.default_logdir
+        kwargs['logdir'] = kwargs.get('logdir', self.default_logdir)
         kwargs['logdir'] = self.project_dir / kwargs['logdir']
         kwargs['num_epochs'] = kwargs.get('num_epochs', 50)
 
@@ -219,7 +331,9 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
             datasets=kwargs.get('datasets', self.get_default_datasets(**kwargs))
         )
         loaders_to_skip = kwargs.get('loader_names_to_skip_in_interpretation', ())
-        interpretation_callback = InterpretationCallback(self.task_flow, tensorboard_converters, loaders_to_skip)
+        interpretation_callback = InterpretationCallback(self.task_flow.get_per_sample_criterion(),
+                                                         tensorboard_converters,
+                                                         loaders_to_skip)
         return interpretation_callback
 
     def get_default_loaders(self, shuffle_train=True,
@@ -267,9 +381,6 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         datasets['infer'] = datasets[kwargs.get('target_loader', 'valid')]
         return datasets
 
-    def batch_to_device(self, batch, device) -> Mapping[str, torch.Tensor]:
-        return any2device(batch, device)
-
     def batch_to_model_device(self, batch) -> Mapping[str, torch.Tensor]:
         return any2device(batch, next(self.model.parameters()).device)
 
@@ -283,12 +394,12 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         if not thresholds_path.exists():
             return model
         tuned_params = torch.load(thresholds_path)
-        self.task_flow.get_decoder().load_tuned(tuned_params)
+        self.task_flow.task.get_decoder().load_tuned(tuned_params)
         return model
 
     def tune(self, loader_name='valid', store=True) -> Dict:
         res = self.load_inference_results()
-        decoder = self.task_flow.get_decoder()
+        decoder = self.task_flow.task.get_decoder()
         tuned_params = decoder.tune(res['logits'][loader_name], res['targets'][loader_name])
         if store:
             out_path = self.project_dir / self.default_logdir / 'tuned_params.pkl'
@@ -297,18 +408,11 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
 
     def load_inference_results(self) -> Dict:
         logdir = self.project_dir / self.default_logdir
-        out_dir = logdir / 'infer'
-        out_dir.mkdir(exist_ok=True)
-        res = {}
-        for key in ['logits', 'targets', 'interpretations']:
-            file_path = out_dir / f'{key}.pkl'
-            if file_path.exists():
-                res[key] = torch.load(out_dir / f'{key}.pkl')
-        return res
+        return load_inference_results_from_directory(logdir)
 
     def load_tuned(self) -> Dict:
         tuned_params = torch.load(self.project_dir / self.default_logdir / 'tuned_params.pkl')
-        self.task_flow.get_decoder().load_tuned(tuned_params)
+        self.task_flow.task.get_decoder().load_tuned(tuned_params)
         return tuned_params
 
     def evaluate(self, loader_name='test'):
@@ -326,7 +430,7 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         torch.save(self.model.state_dict(), out_directory / 'state_dict.pth')
 
 
-def split_already_done(df, project_dir):
+def split_already_done(n: int, project_dir):
     total_len = 0
     for i, split_name in enumerate(['train', 'test', 'val']):
         split_path = project_dir / f'{split_name}_indices.npy'
@@ -334,15 +438,13 @@ def split_already_done(df, project_dir):
             return False
         total_len += len(np.load(split_path))
 
-    if total_len != len(df):
-        return False
-    return True
+    return total_len == n
 
 
-def read_split(project_dir):
+def read_split(runner_dir):
     res = []
     for i, split_name in enumerate(['train', 'test', 'val']):
-        split_path = project_dir / f'{split_name}_indices.npy'
+        split_path = runner_dir / f'{split_name}_indices.npy'
         res.append(np.load(split_path))
     return res
 
@@ -353,9 +455,9 @@ def save_split(project_dir, res):
         np.save(split_path, res[i])
 
 
-def project_split(df, project_dir):
-    if split_already_done(df, project_dir):
-        return read_split(project_dir)
-    res = train_test_val_split(df)
-    save_split(project_dir, res)
+def runner_split(n: int, runner_dir):
+    if split_already_done(n, runner_dir):
+        return read_split(runner_dir)
+    res = train_test_val_split(n)
+    save_split(runner_dir, res)
     return res
