@@ -1,25 +1,13 @@
-import time
 from collections import OrderedDict
 from functools import partial
-from pathlib import Path
 
 import cv2
-import numpy as np
-import pandas as pd
-import torch
-from torch import nn
-from torch.utils.data import DataLoader
+from torch.utils.data import DataLoader, Subset
 
 from dnn_cool.converters import TypeGuesser, ValuesConverter, TaskConverter, Converters
-from dnn_cool.decoders import BoundedRegressionDecoder
-from dnn_cool.project import Project
-from dnn_cool.task_converters import To
-from dnn_cool.task_flow import BoundedRegressionTask, BinaryClassificationTask, ClassificationTask, \
-    MultilabelClassificationTask
-from dnn_cool.utils import torch_split_dataset
-from dnn_cool.value_converters import binary_value_converter, classification_converter, \
-    ImageCoordinatesValuesConverter, MultiLabelValuesConverter
-from dnn_cool.verbosity import Verbosity
+from dnn_cool.tasks import *
+from dnn_cool.utils import split_dataset
+from dnn_cool.value_converters import *
 
 
 def add_random_noise(img):
@@ -176,8 +164,133 @@ def create_df_and_images_tensor(n=int(1e4), cache_file=Path('dnn_cool_synthetic_
     return res
 
 
+def get_synthetic_full_flow(n_shirt_types, n_facial_characteristics) -> TaskFlow:
+    camera_blocked = BinaryClassificationTask('camera_blocked', nn.Linear(256, 1))
+    door_open = BinaryClassificationTask('door_open', nn.Linear(256, 1))
+    person_present = BinaryClassificationTask('person_present', nn.Linear(256, 1))
+    face_x1 = BoundedRegressionTask('face_x1', nn.Linear(256, 1), 64)
+    face_y1 = BoundedRegressionTask('face_y1', nn.Linear(256, 1), 64)
+    face_w = BoundedRegressionTask('face_w', nn.Linear(256, 1), 64)
+    face_h = BoundedRegressionTask('face_h', nn.Linear(256, 1), 64)
+    facial_characteristics = MultilabelClassificationTask('facial_characteristics',
+                                                          nn.Linear(256, n_facial_characteristics))
+    body_x1 = BoundedRegressionTask('body_x1', nn.Linear(256, 1), 64)
+    body_y1 = BoundedRegressionTask('body_y1', nn.Linear(256, 1), 64)
+    body_w = BoundedRegressionTask('body_w', nn.Linear(256, 1), 64)
+    body_h = BoundedRegressionTask('body_h', nn.Linear(256, 1), 64)
+    shirt_type = ClassificationTask('shirt_type', nn.Linear(256, n_shirt_types))
+    door_locked = BinaryClassificationTask('door_locked', nn.Linear(256, 1))
+    leaf_tasks = [
+        camera_blocked,
+        door_open,
+        person_present,
+        face_x1, face_y1, face_w, face_h,
+        facial_characteristics,
+        body_x1, body_y1, body_w, body_h,
+        shirt_type, door_locked
+    ]
+    tasks = Tasks(leaf_tasks)
+
+    @tasks.add_flow
+    def body_regression(flow, x, out):
+        out += flow.body_x1(x.body_localization)
+        out += flow.body_y1(x.body_localization)
+        out += flow.body_w(x.body_localization)
+        out += flow.body_h(x.body_localization)
+        out += flow.shirt_type(x.features)
+        return out
+
+    @tasks.add_flow
+    def face_regression(flow, x, out):
+        out += flow.face_x1(x.face_localization)
+        out += flow.face_y1(x.face_localization)
+        out += flow.face_w(x.face_localization)
+        out += flow.face_h(x.face_localization)
+        out += flow.facial_characteristics(x.features)
+        return out
+
+    @tasks.add_flow
+    def person_regression(flow, x, out):
+        out += flow.face_regression(x)
+        out += flow.body_regression(x)
+        return out
+
+    @tasks.add_flow
+    def full_flow(flow, x, out):
+        out += flow.camera_blocked(x.features)
+        out += flow.door_open(x.features) | (~out.camera_blocked)
+        out += flow.door_locked(x.features) | (~out.door_open)
+        out += flow.person_present(x.features) | out.door_open
+        out += flow.person_regression(x) | out.person_present
+        return out
+
+    return tasks.get_full_flow()
+
+
+class SecurityModule(nn.Module):
+
+    def __init__(self, full_flow):
+        super().__init__()
+        self.seq = nn.Sequential(
+            nn.Conv2d(3, 64, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.Conv2d(64, 128, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.AvgPool2d(2),
+            nn.Conv2d(128, 128, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.Conv2d(128, 256, kernel_size=5),
+            nn.AvgPool2d(2),
+            nn.ReLU(inplace=True),
+        )
+
+        self.features_seq = nn.Sequential(
+            nn.Conv2d(256, 256, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.Conv2d(256, 256, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.AdaptiveAvgPool2d(1),
+            nn.Flatten(),
+        )
+
+        self.face_localization_seq = nn.Sequential(
+            nn.Conv2d(256, 256, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.Conv2d(256, 256, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.AdaptiveAvgPool2d(1),
+            nn.Flatten(),
+        )
+
+        self.body_localization_seq = nn.Sequential(
+            nn.Conv2d(256, 256, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.Conv2d(256, 256, kernel_size=5),
+            nn.ReLU(inplace=True),
+            nn.AdaptiveAvgPool2d(1),
+            nn.Flatten(),
+        )
+
+        self.flow_module = full_flow.torch()
+
+    def forward(self, x):
+        res = {}
+        common = self.seq(x['syn_img'])
+        res['features'] = self.features_seq(common)
+        res['face_localization'] = self.face_localization_seq(common)
+        res['body_localization'] = self.body_localization_seq(common)
+        res['gt'] = x.get('gt')
+        return self.flow_module(res)
+
+
 def synthetic_dataset_preparation(n=int(1e4), perform_conversion=True):
     imgs, df = create_df_and_images_tensor(n)
+    multilabel_converter = MultiLabelValuesConverter()
+    n_shirt_types = classification_converter(df['shirt_type']).max().item() + 1
+    n_facial_characteristics = multilabel_converter(df['facial_characteristics']).shape[1]
+
+    full_flow = get_synthetic_full_flow(n_shirt_types, n_facial_characteristics)
+
     output_col = ['camera_blocked', 'door_open', 'person_present', 'door_locked',
                   'face_x1', 'face_y1', 'face_w', 'face_h',
                   'facial_characteristics',
@@ -210,66 +323,29 @@ def synthetic_dataset_preparation(n=int(1e4), perform_conversion=True):
 
     task_converter = TaskConverter()
 
-    task_converter.type_mapping['binary'] = To(BinaryClassificationTask,
-                                               module_supplier=partial(nn.Linear, in_features=256, out_features=1))
-    task_converter.type_mapping['continuous'] = To(BoundedRegressionTask,
-                                                   module_supplier=partial(nn.Linear, in_features=256, out_features=1),
-                                                   decoder_supplier=partial(BoundedRegressionDecoder, scale=64))
-    n_classes = classification_converter(df['shirt_type']).max().item() + 1
-    task_converter.type_mapping['category'] = To(ClassificationTask,
-                                                 module_supplier=partial(nn.Linear,
-                                                                         in_features=256, out_features=n_classes))
-    n_classes = multilabel_converter(df['facial_characteristics']).shape[1]
-    task_converter.type_mapping['multilabel'] = To(MultilabelClassificationTask,
-                                                   module_supplier=partial(nn.Linear,
-                                                                           in_features=256, out_features=n_classes))
+    task_converter.type_mapping['binary'] = BinaryClassificationTaskForDevelopment
+    task_converter.type_mapping['continuous'] = BoundedRegressionTaskForDevelopment
+    task_converter.type_mapping['category'] = ClassificationTaskForDevelopment
+    task_converter.type_mapping['multilabel'] = MultilabelClassificationTaskForDevelopment
 
-    converters = Converters()
+    converters = Converters(project_dir=Path('./security_project'))
     converters.task = task_converter
     converters.type = type_guesser
     converters.values = values_converter
 
-    df = df if perform_conversion else None
-    project = Project(df, input_col='syn_img', output_col=output_col, converters=converters,
-                      project_dir='./security_project',
-                      verbosity=Verbosity.BASIC_STATS)
+    full_flow_for_development = converters.create_task_flow_for_development(df, input_col='syn_img',
+                                                                            output_col=output_col,
+                                                                            task_flow=full_flow)
+    # df = df if perform_conversion else None
+    # project = Project(df, input_col='syn_img', output_col=output_col, converters=converters,
+    #                   project_dir='./security_project',
+    #                   verbosity=Verbosity.BASIC_STATS)
 
-    @project.add_flow
-    def face_regression(flow, x, out):
-        out += flow.face_x1(x.face_localization)
-        out += flow.face_y1(x.face_localization)
-        out += flow.face_w(x.face_localization)
-        out += flow.face_h(x.face_localization)
-        out += flow.facial_characteristics(x.features)
-        return out
-
-    @project.add_flow
-    def body_regression(flow, x, out):
-        out += flow.body_x1(x.body_localization)
-        out += flow.body_y1(x.body_localization)
-        out += flow.body_w(x.body_localization)
-        out += flow.body_h(x.body_localization)
-        out += flow.shirt_type(x.features)
-        return out
-
-    @project.add_flow
-    def person_regression(flow, x, out):
-        out += flow.face_regression(x)
-        out += flow.body_regression(x)
-        return out
-
-    @project.add_flow
-    def full_flow(flow, x, out):
-        out += flow.camera_blocked(x.features)
-        out += flow.door_open(x.features) | (~out.camera_blocked)
-        out += flow.door_locked(x.features) | (~out.door_open)
-        out += flow.person_present(x.features) | out.door_open
-        out += flow.person_regression(x) | out.person_present
-        return out
-
-    dataset = project.get_full_flow().get_dataset()
+    dataset = full_flow_for_development.get_dataset()
     if perform_conversion:
-        train_dataset, val_dataset = torch_split_dataset(dataset, random_state=42)
+        train_indices, val_indices = split_dataset(len(dataset), random_state=42)
+        train_dataset = Subset(dataset, train_indices)
+        val_dataset = Subset(dataset, val_indices)
         train_loader = DataLoader(train_dataset, batch_size=32 * torch.cuda.device_count(), shuffle=True)
         val_loader = DataLoader(val_dataset, batch_size=32 * torch.cuda.device_count(), shuffle=False)
         nested_loaders = OrderedDict({
@@ -281,70 +357,15 @@ def synthetic_dataset_preparation(n=int(1e4), perform_conversion=True):
         train_dataset = None
         val_dataset = None
 
-    class SecurityModule(nn.Module):
-
-        def __init__(self):
-            super().__init__()
-            self.seq = nn.Sequential(
-                nn.Conv2d(3, 64, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.Conv2d(64, 128, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.AvgPool2d(2),
-                nn.Conv2d(128, 128, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.Conv2d(128, 256, kernel_size=5),
-                nn.AvgPool2d(2),
-                nn.ReLU(inplace=True),
-            )
-
-            self.features_seq = nn.Sequential(
-                nn.Conv2d(256, 256, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.Conv2d(256, 256, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.AdaptiveAvgPool2d(1),
-                nn.Flatten(),
-            )
-
-            self.face_localization_seq = nn.Sequential(
-                nn.Conv2d(256, 256, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.Conv2d(256, 256, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.AdaptiveAvgPool2d(1),
-                nn.Flatten(),
-            )
-
-            self.body_localization_seq = nn.Sequential(
-                nn.Conv2d(256, 256, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.Conv2d(256, 256, kernel_size=5),
-                nn.ReLU(inplace=True),
-                nn.AdaptiveAvgPool2d(1),
-                nn.Flatten(),
-            )
-
-            self.flow_module = full_flow.torch()
-
-        def forward(self, x):
-            res = {}
-            common = self.seq(x['syn_img'])
-            res['features'] = self.features_seq(common)
-            res['face_localization'] = self.face_localization_seq(common)
-            res['body_localization'] = self.body_localization_seq(common)
-            res['gt'] = x.get('gt')
-            return self.flow_module(res)
-
-    model = SecurityModule()
+    model = SecurityModule(full_flow)
     datasets = {
         'train': train_dataset,
         'valid': val_dataset,
         'infer': val_dataset
     }
-    children = full_flow.get_all_children()
+    children = full_flow_for_development.get_all_children()
     shirt_type = children['person_regression.body_regression.shirt_type']
     shirt_type.top_k = 10
     shirt_type.class_names = ['blue', 'red', 'yellow', 'cyan', 'magenta', 'green', 'black']
     children['person_regression.face_regression.facial_characteristics'].class_names = ['red', 'green', 'blue']
-    return model, nested_loaders, datasets, project
+    return model, nested_loaders, datasets, full_flow_for_development, converters.tensorboard_converters
