@@ -1,6 +1,10 @@
+from pathlib import Path
+from typing import Sized, Union
+
 import numpy as np
 import torch
 from sklearn.model_selection import train_test_split
+from torch import nn
 from torch.utils.data import Dataset, Subset
 from torchvision import transforms
 
@@ -18,15 +22,15 @@ def torch_split_dataset(dataset, test_size=0.2, random_state=None):
     return train_dataset, test_dataset
 
 
-def split_dataset(dataset, test_size=0.2, random_state=None):
-    X, y = np.arange(len(dataset)), np.arange(len(dataset))
+def split_dataset(n: int, test_size=0.2, random_state=None):
+    X, y = np.arange(n), np.arange(n)
     train_indices, test_indices, _, _ = train_test_split(X, y, test_size=test_size, random_state=random_state)
     return train_indices, test_indices
 
 
-def train_test_val_split(df, random_state=None):
-    train_indices, val_indices = split_dataset(df, test_size=0.2, random_state=random_state)
-    rel_test_indices, rel_val_indices = split_dataset(val_indices, test_size=0.5, random_state=random_state)
+def train_test_val_split(n: int, random_state=None):
+    train_indices, val_indices = split_dataset(n, test_size=0.2, random_state=random_state)
+    rel_test_indices, rel_val_indices = split_dataset(len(val_indices), test_size=0.5, random_state=random_state)
     return train_indices, val_indices[rel_test_indices], val_indices[rel_val_indices]
 
 
@@ -104,3 +108,31 @@ class ImageNetNormalizer:
         X = self.normalize(X)
         return X
 
+
+class Values:
+
+    def __init__(self, keys, values, types):
+        assert len(keys) == len(values)
+        self.keys = keys
+        self.values = values
+        self.types = types
+
+    def __getitem__(self, item):
+        res = {}
+        for i, key in enumerate(self.keys):
+            res[key] = self.values[i][item]
+        return res
+
+    def __len__(self):
+        return len(self.values[0])
+
+
+def load_model_from_export(model, full_flow, out_directory: Union[str, Path]) -> nn.Module:
+    out_directory = Path(out_directory)
+    model.load_state_dict(torch.load(out_directory / 'state_dict.pth'))
+    thresholds_path = out_directory / 'tuned_params.pkl'
+    if not thresholds_path.exists():
+        return model
+    tuned_params = torch.load(thresholds_path)
+    full_flow.get_decoder().load_tuned(tuned_params)
+    return model
